/***********************************************************************
 *         
 *         S A I T E K   T H R O T T L E   Q U A D R A N T 
 *         
 *        restored with the Arduino Leonardo (Arduino Micro)
 *         
 *         Using ArduinoJoystickLibrary by Matthew Heironimus
 *        https://github.com/MHeironimus/ArduinoJoystickLibrary
 **********************************************************************/
#include "Joystick.h"
/*--------------------------------------------------------
  "THE BEER-WARE LICENSE" (Revision 42):
  Alex Kostyuk wrote this code. As long as you retain this 
  notice you can do whatever you want with this stuff. 
  If we meet some day, and you think this stuff is worth it, 
  you can buy me a beer in return.
----------------------------------------------------------*/

Joystick_ Joystick(JOYSTICK_DEFAULT_REPORT_ID, 
  JOYSTICK_TYPE_MULTI_AXIS, 32, 0,
  true, true, true, true, true, true,
  false, false, false, false, false);


void setup()
{
    bits_init();
    moving_average_filter_init();

    Joystick.setXAxisRange(0, 255);
    Joystick.setYAxisRange(0, 255);
    Joystick.setZAxisRange(0, 255);
    Joystick.begin(false);
}

void loop(void)
{
    static int USB_timer = 0;
    discrete_task();
 
    if(++USB_timer == 50)
    {
        USB_timer = 0;
        Joystick.setXAxis(moving_average_filter(0,analogRead(A0))); 
        Joystick.setYAxis(moving_average_filter(1,analogRead(A1)));
        Joystick.setZAxis(moving_average_filter(2,analogRead(A2)));
        Joystick.sendState();
    }
}

//-------------------------------------------
//            Moving average filter
//-------------------------------------------
#define ANALOG_FIFO_SIZE (32)
#define CHANNELS_NUMBER  (3)
typedef struct
{
  int   push_indx;
  int   pop_indx;
  long buff[ANALOG_FIFO_SIZE];
} SIMPLE_FIFO;
SIMPLE_FIFO analog_fifo[CHANNELS_NUMBER];
short fifo_push(SIMPLE_FIFO *fifo, short data);
short fifo_pop(SIMPLE_FIFO *fifo, short *data);
char lever_state[3];

void moving_average_filter_init(void)
{
   memset(analog_fifo, 0, sizeof(analog_fifo));
}

short moving_average_filter(int ch, short in)
{
static short sum[CHANNELS_NUMBER] = {0,0,0};
short tmp = in;
short prev;
     sum[ch] += in;
     if(!fifo_push(&analog_fifo[ch], in))
     {
       fifo_pop(&analog_fifo[ch], &prev);
       fifo_push(&analog_fifo[ch], in);
       sum[ch] -= prev;
       tmp = (sum[ch] / (ANALOG_FIFO_SIZE-1)) / 3;
       
       if(tmp < 35)
           lever_state[ch] = 0;           
       else if(tmp > 45)
           lever_state[ch] = 1;           
       
       tmp -= 50;
       if(tmp > 255)
          tmp = 255;
       if(tmp < 0)
          tmp = 0;     
       return tmp;
     } 
  
         
return 0;
}

short fifo_push(SIMPLE_FIFO *fifo, short data)
{
    int indx;
    indx = fifo->push_indx + 1;
    indx = indx % ANALOG_FIFO_SIZE;

    if (indx == fifo->pop_indx)
        return 0; // fifo is full
    fifo->buff[indx] = data;
    fifo->push_indx = indx;
    return 1;
}

short fifo_pop(SIMPLE_FIFO *fifo, short *data)
{
    int indx;
    if (fifo->push_indx == fifo->pop_indx)
        return 0; // fifo is empty
    indx = fifo->pop_indx + 1;
    indx = indx % ANALOG_FIFO_SIZE;
    *data = fifo->buff[indx];
    fifo->pop_indx = indx;
    return 1;
}

//-------------------------------------------
//               Discrete task
//-------------------------------------------
#define BITS_MAXIMUM   12 /*ITEMS_MAX*/
int shift = 0;

#define FLAG_INVERTED   (1<<0)
#define FLAG_ENABLED    (1<<1)

enum {
  PUSH_BUTTON = 0,
  TOGGLE_SWITCH,
  ROTARY_SWITCH_POLE,
  ENCODER_A,
  ENCODER_B,
  SHIFT
};

typedef struct {
int counter;
char signal;
int  max;
char evcode;
char flag;
char mode;
char pin;
char emitted_code_rising[3];
char emitted_code_falling[3];
int pulse_timer;
int pulse_timer_value;
} BITS_DESCRIPTION;

BITS_DESCRIPTION  bits[BITS_MAXIMUM];
 
#define input_init(x,y,z,m,r1,f1,r2,f2,r3,f3, t)   do {\
bits[x].counter = 0;\
bits[x].signal = 0;\
bits[x].max = y;\
bits[x].flag = (FLAG_ENABLED | FLAG_INVERTED);\
bits[x].pin = z;\
pinMode(z,INPUT_PULLUP);\
bits[x].mode = m;\
bits[x].emitted_code_rising[0] = r1;\
bits[x].emitted_code_rising[1] = r2;\
bits[x].emitted_code_rising[2] = r3;\
bits[x].emitted_code_falling[0] = f1;\
bits[x].emitted_code_falling[1] = f3;\
bits[x].emitted_code_falling[2] = f3;\
bits[x].pulse_timer_value = t;\
} while(0)

void bits_init(void)
{
  int i;
  memset(bits,0,sizeof(bits));
// buttons  
#define DEBOUNCE_DELAY   250
input_init(0,DEBOUNCE_DELAY,2, PUSH_BUTTON,   0,-1, -1, -1, -1,-1, 0);
input_init(1,DEBOUNCE_DELAY,3, PUSH_BUTTON,   1,-1, -1,-1, -1,-1,  0);
input_init(2,DEBOUNCE_DELAY,4, PUSH_BUTTON,   2,-1, -1,-1, -1,-1,  0);
input_init(3,DEBOUNCE_DELAY,5, PUSH_BUTTON,   3,-1, -1,-1, -1,-1,  0);
input_init(4,DEBOUNCE_DELAY,6, PUSH_BUTTON,   4,-1, -1,-1, -1,-1,  0);
input_init(5,DEBOUNCE_DELAY,7, PUSH_BUTTON,   5,-1, -1,-1, -1,-1,  0);
input_init(6,DEBOUNCE_DELAY,8, PUSH_BUTTON,   6,-1, -1,-1, -1,-1,  0);
input_init(7,DEBOUNCE_DELAY,9, PUSH_BUTTON,   7,-1, -1,-1, -1,-1,  0);
input_init(8,DEBOUNCE_DELAY,10, PUSH_BUTTON,  8,-1, -1,-1, -1,-1,  0);
// levers limit switches (virtual signals)
input_init(9,DEBOUNCE_DELAY, 0, PUSH_BUTTON,  9,-1, -1,-1, -1,-1,  0);
input_init(10,DEBOUNCE_DELAY,0, PUSH_BUTTON, 10,-1, -1,-1, -1,-1,  0);
input_init(11,DEBOUNCE_DELAY,0, PUSH_BUTTON, 11,-1, -1,-1, -1,-1,  0);
}

void discrete_task( void)
{
    int bit,flag;
    int bit_state;
    int i;

    for(i=0;i<BITS_MAXIMUM;i++)
    {
        if(bits[i].pulse_timer)
        {
            if(--bits[i].pulse_timer == 0)
            {
                switch(bits[i].mode)
                {
                    case PUSH_BUTTON:
                    
                    break;

                    case ENCODER_A:
                        if(bits[i+1].signal)
                        {
                            Joystick.releaseButton(bits[i].emitted_code_rising[0]);
                        } else
                        {
                            Joystick.releaseButton(bits[i+1].emitted_code_rising[0]);
                        }
                    break;

                    case ENCODER_B:
                    
                    break;

                    case TOGGLE_SWITCH:
                        Joystick.pressButton(bits[i].emitted_code_rising[0]);
                        Joystick.releaseButton(bits[i].emitted_code_falling[0]);
                    break;

                }
            }
        }

        if(bits[i].flag & FLAG_ENABLED)
        {
            if(bits[i].pin) // regular signal
                bit_state = digitalRead(bits[i].pin);
            else // lever limit switch virtual signal
                bit_state = lever_state[i-9];

            flag = bits[i].flag;
            bit = (flag & FLAG_INVERTED) ^ bit_state;

            if(bit)
            {
                if(bits[i].counter < bits[i].max)
                {
                    if(++(bits[i].counter) >= bits[i].max)
                    {
                        if(bits[i].signal == 0)
                        {
                            //front __0__/--1-- 
                            switch(bits[i].mode)
                            {
                                case PUSH_BUTTON:
                                    Joystick.pressButton(bits[i].emitted_code_rising[shift]);
                                break;

                                case ENCODER_A:
                                    if(bits[i+1].signal)
                                    {
                                        Joystick.pressButton(bits[i].emitted_code_rising[shift]);
                                    } else
                                    {
                                        Joystick.pressButton(bits[i+1].emitted_code_rising[shift]);
                                    }
                                break;

                                case ENCODER_B:
                                break;

                                case TOGGLE_SWITCH:
                                    bits[i].pulse_timer = bits[i].pulse_timer_value;
                                    Joystick.pressButton(bits[i].emitted_code_rising[0]);
                                    Joystick.releaseButton(bits[i].emitted_code_falling[0]);
                                break;

                                case SHIFT:
                                    shift = bits[i].emitted_code_rising[0];
                                break;

                            }
                            bits[i].signal = 1;
                        }

                    }
                }
            }
            else
            {
                if(bits[i].counter > 0)
                {
                    if(--(bits[i].counter) <= 0)
                    {
                        if(bits[i].signal)
                        {
                            // edge  --1--\___0___ 
                            switch(bits[i].mode)
                            {
                                case PUSH_BUTTON:
                                    Joystick.releaseButton(bits[i].emitted_code_rising[shift]);
                                break;

                                case ENCODER_A:
                                    Joystick.pressButton(bits[i].emitted_code_rising[shift]);
                                    Joystick.pressButton(bits[i+1].emitted_code_rising[shift]);
                                break;

                                case ENCODER_B:
                                
                                break;

                                case TOGGLE_SWITCH:
                                    bits[i].pulse_timer = bits[i].pulse_timer_value;
                                    Joystick.releaseButton(bits[i].emitted_code_falling[0]);
                                    Joystick.pressButton(bits[i].emitted_code_rising[0]);
                                break;

                                case SHIFT:
                                    shift = bits[i].emitted_code_falling[0];
                                break;
                            }
                            bits[i].signal = 0;
                        }
                    }
                }
            }
        }
    }
}










